@@ -14,16 +14,22 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 from contextlib import contextmanager
 import errno
+import functools
 import logging
 import os
+from pathlib import Path
 import threading
 import time
-
-import six
+from typing import Callable
+from typing import Optional
+from typing import Union
 
 from fasteners import _utils
+from fasteners.process_mechanism import _interprocess_mechanism
+from fasteners.process_mechanism import _interprocess_reader_writer_mechanism
 
 LOG = logging.getLogger(__name__)
 
@@ -49,24 +55,25 @@ def _ensure_tree(path):
         return True
 
 
-class _InterProcessLock(object):
+class InterProcessLock:
     """An interprocess lock."""
 
-    MAX_DELAY = 0.1
-    """
-    Default maximum delay we will wait to try to acquire the lock (when
-    it's busy/being held by another process).
-    """
+    MAX_DELAY = 0.1  # For backwards compatibility
+    DELAY_INCREMENT = 0.01  # For backwards compatibility
 
-    DELAY_INCREMENT = 0.01
-    """
-    Default increment we will use (up to max delay) after each attempt before
-    next attempt to acquire the lock. For example if 3 attempts have been made
-    the calling thread will sleep (0.01 * 3) before the next attempt to
-    acquire the lock (and repeat).
-    """
-
-    def __init__(self, path, sleep_func=time.sleep, logger=None):
+    def __init__(self,
+                 path: Union[Path, str],
+                 sleep_func: Callable[[float], None] = time.sleep,
+                 logger: Optional[logging.Logger] = None):
+        """
+        args:
+            path:
+                Path to the file that will be used for locking.
+            sleep_func:
+                Optional function to use for sleeping.
+            logger:
+                Optional logger to use for logging.
+        """
         self.lockfile = None
         self.path = _utils.canonicalize_path(path)
         self.acquired = False
@@ -106,25 +113,27 @@ class _InterProcessLock(object):
         if self.lockfile is None or self.lockfile.closed:
             self.lockfile = open(self.path, 'a')
 
-    def acquire(self, blocking=True,
-                delay=DELAY_INCREMENT, max_delay=MAX_DELAY,
-                timeout=None):
-        """Attempt to acquire the given lock.
+    def acquire(self,
+                blocking: bool = True,
+                delay: float = 0.01,
+                max_delay: float = 0.1,
+                timeout: Optional[float] = None) -> bool:
+        """Attempt to acquire the lock.
 
-        :param blocking: whether to wait forever to try to acquire the lock
-        :type blocking: bool
-        :param delay: when blocking this is the delay time in seconds that
-                      will be added after each failed acquisition
-        :type delay: int/float
-        :param max_delay: the maximum delay to have (this limits the
-                          accumulated delay(s) added after each failed
-                          acquisition)
-        :type max_delay: int/float
-        :param timeout: an optional timeout (limits how long blocking
-                        will occur for)
-        :type timeout: int/float
-        :returns: whether or not the acquisition succeeded
-        :rtype: bool
+        Args:
+            blocking:
+                Whether to wait to try to acquire the lock.
+            delay:
+                When `blocking`, starting delay as well as the delay increment
+                (in seconds).
+            max_delay:
+                When `blocking` the maximum delay in between attempts to
+                acquire (in seconds).
+            timeout:
+                When `blocking`, maximal waiting time (in seconds).
+
+        Returns:
+            whether or not the acquisition succeeded
         """
         if delay < 0:
             raise ValueError("Delay must be greater than or equal to zero")
@@ -139,7 +148,6 @@ class _InterProcessLock(object):
         with watch:
             gotten = r(self._try_acquire, blocking, watch)
         if not gotten:
-            self.acquired = False
             return False
         else:
             self.acquired = True
@@ -157,7 +165,7 @@ class _InterProcessLock(object):
     def __enter__(self):
         gotten = self.acquire()
         if not gotten:
-            # This shouldn't happen, but just incase...
+            # This shouldn't happen, but just in case...
             raise threading.ThreadError("Unable to acquire a file lock"
                                         " on `%s` (when used as a"
                                         " context manager)" % self.path)
@@ -166,13 +174,13 @@ class _InterProcessLock(object):
     def release(self):
         """Release the previously acquired lock."""
         if not self.acquired:
-            raise threading.ThreadError("Unable to release an unacquired"
-                                        " lock")
+            raise threading.ThreadError("Unable to release an unaquired lock")
         try:
             self.unlock()
-        except IOError:
-            self.logger.exception("Could not unlock the acquired lock opened"
-                                  " on `%s`", self.path)
+        except Exception as e:
+            msg = "Could not unlock the acquired lock opened on `%s`", self.path
+            self.logger.exception(msg)
+            raise threading.ThreadError(msg) from e
         else:
             self.acquired = False
             try:
@@ -189,50 +197,70 @@ class _InterProcessLock(object):
         self.release()
 
     def exists(self):
-        """Checks if the path that this lock exists at actually exists."""
         return os.path.exists(self.path)
 
     def trylock(self):
-        self._trylock(self.lockfile)
+        _interprocess_mechanism.trylock(self.lockfile)
 
     def unlock(self):
-        self._unlock(self.lockfile)
-
-    @staticmethod
-    def _trylock(lockfile):
-        raise NotImplementedError()
-
-    @staticmethod
-    def _unlock(lockfile):
-        raise NotImplementedError()
+        _interprocess_mechanism.unlock(self.lockfile)
 
 
-class _InterProcessReaderWriterLock(object):
+class InterProcessReaderWriterLock:
     """An interprocess readers writer lock."""
 
-    MAX_DELAY = 0.1
-    """
-    Default maximum delay we will wait to try to acquire the lock (when
-    it's busy/being held by another process).
-    """
+    MAX_DELAY = 0.1  # for backwards compatibility
+    DELAY_INCREMENT = 0.01  # for backwards compatibility
 
-    DELAY_INCREMENT = 0.01
-    """
-    Default increment we will use (up to max delay) after each attempt before
-    next attempt to acquire the lock. For example if 3 attempts have been made
-    the calling thread will sleep (0.01 * 3) before the next attempt to
-    acquire the lock (and repeat).
-    """
-
-    def __init__(self, path, sleep_func=time.sleep, logger=None):
+    def __init__(self,
+                 path: Union[Path, str],
+                 sleep_func: Callable[[float], None] = time.sleep,
+                 logger: Optional[logging.Logger] = None):
+        """
+        Args:
+            path:
+                Path to the file that will be used for locking.
+            sleep_func:
+                Optional function to use for sleeping.
+            logger:
+                Optional logger to use for logging.
+        """
         self.lockfile = None
         self.path = _utils.canonicalize_path(path)
         self.sleep_func = sleep_func
         self.logger = _utils.pick_first_not_none(logger, LOG)
 
+    @contextmanager
+    def read_lock(self, delay=0.01, max_delay=0.1):
+        """Context manager that grans a read lock"""
+
+        self.acquire_read_lock(blocking=True, delay=delay,
+                               max_delay=max_delay, timeout=None)
+        try:
+            yield
+        finally:
+            self.release_read_lock()
+
+    @contextmanager
+    def write_lock(self, delay=0.01, max_delay=0.1):
+        """Context manager that grans a write lock"""
+
+        gotten = self.acquire_write_lock(blocking=True, delay=delay,
+                                         max_delay=max_delay, timeout=None)
+
+        if not gotten:
+            # This shouldn't happen, but just in case...
+            raise threading.ThreadError("Unable to acquire a file lock"
+                                        " on `%s` (when used as a"
+                                        " context manager)" % self.path)
+        try:
+            yield
+        finally:
+            self.release_write_lock()
+
     def _try_acquire(self, blocking, watch, exclusive):
         try:
-            gotten = self._trylock(self.lockfile, exclusive)
+            gotten = _interprocess_reader_writer_mechanism.trylock(self.lockfile, exclusive)
         except Exception as e:
             raise threading.ThreadError(
                 "Unable to acquire lock on {} due to {}!".format(self.path, e))
@@ -253,56 +281,58 @@ class _InterProcessReaderWriterLock(object):
                 self.logger.log(_utils.BLATHER,
                                 'Created lock base path `%s`', basedir)
         if self.lockfile is None:
-            self.lockfile = self._get_handle(self.path)
-
-    def acquire_read_lock(self, blocking=True,
-                          delay=DELAY_INCREMENT, max_delay=MAX_DELAY,
-                          timeout=None):
+            self.lockfile = _interprocess_reader_writer_mechanism.get_handle(self.path)
 
+    def acquire_read_lock(self,
+                          blocking: bool = True,
+                          delay: float = 0.01,
+                          max_delay: float = 0.1,
+                          timeout: float = None) -> bool:
         """Attempt to acquire a reader's lock.
 
-        :param blocking: whether to wait forever to try to acquire the lock
-        :type blocking: bool
-        :param delay: when blocking this is the delay time in seconds that
-                      will be added after each failed acquisition
-        :type delay: int/float
-        :param max_delay: the maximum delay to have (this limits the
-                          accumulated delay(s) added after each failed
-                          acquisition)
-        :type max_delay: int/float
-        :param timeout: an optional timeout (limits how long blocking
-                        will occur for)
-        :type timeout: int/float
-        :returns: whether or not the acquisition succeeded
-        :rtype: bool
+        Args:
+            blocking:
+                Whether to wait to try to acquire the lock.
+            delay:
+                When `blocking`, starting delay as well as the delay increment
+                (in seconds).
+            max_delay:
+                When `blocking` the maximum delay in between attempts to
+                acquire (in seconds).
+            timeout:
+                When `blocking`, maximal waiting time (in seconds).
+
+        Returns:
+            whether or not the acquisition succeeded
         """
         return self._acquire(blocking, delay, max_delay, timeout, exclusive=False)
 
-    def acquire_write_lock(self, blocking=True,
-                           delay=DELAY_INCREMENT, max_delay=MAX_DELAY,
-                           timeout=None):
-
+    def acquire_write_lock(self,
+                           blocking: bool = True,
+                           delay: float = 0.01,
+                           max_delay: float = 0.1,
+                           timeout: float = None) -> bool:
         """Attempt to acquire a writer's lock.
 
-        :param blocking: whether to wait forever to try to acquire the lock
-        :type blocking: bool
-        :param delay: when blocking this is the delay time in seconds that
-                      will be added after each failed acquisition
-        :type delay: int/float
-        :param max_delay: the maximum delay to have (this limits the
-                          accumulated delay(s) added after each failed
-                          acquisition)
-        :type max_delay: int/float
-        :param timeout: an optional timeout (limits how long blocking
-                        will occur for)
-        :type timeout: int/float
-        :returns: whether or not the acquisition succeeded
-        :rtype: bool
+        Args:
+            blocking:
+                Whether to wait to try to acquire the lock.
+            delay:
+                When `blocking`, starting delay as well as the delay increment
+                (in seconds).
+            max_delay:
+                When `blocking` the maximum delay in between attempts to
+                acquire (in seconds).
+            timeout:
+                When `blocking`, maximal waiting time (in seconds).
+
+        Returns:
+            whether or not the acquisition succeeded
         """
         return self._acquire(blocking, delay, max_delay, timeout, exclusive=True)
 
     def _acquire(self, blocking=True,
-                 delay=DELAY_INCREMENT, max_delay=MAX_DELAY,
+                 delay=0.01, max_delay=0.1,
                  timeout=None, exclusive=True):
 
         if delay < 0:
@@ -328,13 +358,13 @@ class _InterProcessReaderWriterLock(object):
 
     def _do_close(self):
         if self.lockfile is not None:
-            self._close_handle(self.lockfile)
+            _interprocess_reader_writer_mechanism.close_handle(self.lockfile)
             self.lockfile = None
 
     def release_write_lock(self):
         """Release the writer's lock."""
         try:
-            self._unlock(self.lockfile)
+            _interprocess_reader_writer_mechanism.unlock(self.lockfile)
         except IOError:
             self.logger.exception("Could not unlock the acquired lock opened"
                                   " on `%s`", self.path)
@@ -352,7 +382,7 @@ class _InterProcessReaderWriterLock(object):
     def release_read_lock(self):
         """Release the reader's lock."""
         try:
-            self._unlock(self.lockfile)
+            _interprocess_reader_writer_mechanism.unlock(self.lockfile)
         except IOError:
             self.logger.exception("Could not unlock the acquired lock opened"
                                   " on `%s`", self.path)
@@ -367,172 +397,18 @@ class _InterProcessReaderWriterLock(object):
                                 "Unlocked and closed file lock open on"
                                 " `%s`", self.path)
 
-    @contextmanager
-    def write_lock(self, delay=DELAY_INCREMENT, max_delay=MAX_DELAY):
 
-        gotten = self.acquire_write_lock(blocking=True, delay=delay,
-                                         max_delay=max_delay, timeout=None)
-
-        if not gotten:
-            # This shouldn't happen, but just in case...
-            raise threading.ThreadError("Unable to acquire a file lock"
-                                        " on `%s` (when used as a"
-                                        " context manager)" % self.path)
-        try:
-            yield
-        finally:
-            self.release_write_lock()
-
-    @contextmanager
-    def read_lock(self, delay=DELAY_INCREMENT, max_delay=MAX_DELAY):
-
-        self.acquire_read_lock(blocking=True, delay=delay,
-                               max_delay=max_delay, timeout=None)
-        try:
-            yield
-        finally:
-            self.release_read_lock()
-
-    @staticmethod
-    def _trylock(lockfile, exclusive):
-        raise NotImplementedError()
-
-    @staticmethod
-    def _unlock(lockfile):
-        raise NotImplementedError()
-
-    @staticmethod
-    def _get_handle(path):
-        raise NotImplementedError()
-
-    @staticmethod
-    def _close_handle(lockfile):
-        raise NotImplementedError()
-
-
-class _WindowsLock(_InterProcessLock):
-    """Interprocess lock implementation that works on windows systems."""
-
-    @staticmethod
-    def _trylock(lockfile):
-        fileno = lockfile.fileno()
-        msvcrt.locking(fileno, msvcrt.LK_NBLCK, 1)
-
-    @staticmethod
-    def _unlock(lockfile):
-        fileno = lockfile.fileno()
-        msvcrt.locking(fileno, msvcrt.LK_UNLCK, 1)
-
-
-class _FcntlLock(_InterProcessLock):
-    """Interprocess lock implementation that works on posix systems."""
-
-    @staticmethod
-    def _trylock(lockfile):
-        fcntl.lockf(lockfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
-
-    @staticmethod
-    def _unlock(lockfile):
-        fcntl.lockf(lockfile, fcntl.LOCK_UN)
-
-
-class _WindowsInterProcessReaderWriterLock(_InterProcessReaderWriterLock):
-    """Interprocess readers writer lock implementation that works on windows
-    systems."""
-
-    @staticmethod
-    def _trylock(lockfile, exclusive):
-
-        if exclusive:
-            flags = win32con.LOCKFILE_FAIL_IMMEDIATELY | win32con.LOCKFILE_EXCLUSIVE_LOCK
-        else:
-            flags = win32con.LOCKFILE_FAIL_IMMEDIATELY
-
-        handle = msvcrt.get_osfhandle(lockfile.fileno())
-        ok = win32file.LockFileEx(handle, flags, 0, 1, 0, win32file.pointer(pywintypes.OVERLAPPED()))
-        if ok:
-            return True
-        else:
-            last_error = win32file.GetLastError()
-            if last_error == win32file.ERROR_LOCK_VIOLATION:
-                return False
-            else:
-                raise OSError(last_error)
-
-    @staticmethod
-    def _unlock(lockfile):
-        handle = msvcrt.get_osfhandle(lockfile.fileno())
-        ok = win32file.UnlockFileEx(handle, 0, 1, 0, win32file.pointer(pywintypes.OVERLAPPED()))
-        if not ok:
-            raise OSError(win32file.GetLastError())
-
-    @staticmethod
-    def _get_handle(path):
-        return open(path, 'a+')
-
-    @staticmethod
-    def _close_handle(lockfile):
-        lockfile.close()
-
-
-class _FcntlInterProcessReaderWriterLock(_InterProcessReaderWriterLock):
-    """Interprocess readers writer lock implementation that works on posix
-    systems."""
-
-    @staticmethod
-    def _trylock(lockfile, exclusive):
-
-        if exclusive:
-            flags = fcntl.LOCK_EX | fcntl.LOCK_NB
-        else:
-            flags = fcntl.LOCK_SH | fcntl.LOCK_NB
-
-        try:
-            fcntl.lockf(lockfile, flags)
-            return True
-        except (IOError, OSError) as e:
-            if e.errno in (errno.EACCES, errno.EAGAIN):
-                return False
-            else:
-                raise e
-
-    @staticmethod
-    def _unlock(lockfile):
-        fcntl.lockf(lockfile, fcntl.LOCK_UN)
-
-    @staticmethod
-    def _get_handle(path):
-        return open(path, 'a+')
-
-    @staticmethod
-    def _close_handle(lockfile):
-        lockfile.close()
-
-
-if os.name == 'nt':
-    import msvcrt
-    import fasteners.pywin32.pywintypes as pywintypes
-    import fasteners.pywin32.win32con as win32con
-    import fasteners.pywin32.win32file as win32file
-
-    InterProcessLock = _WindowsLock
-    InterProcessReaderWriterLock = _WindowsInterProcessReaderWriterLock
-
-else:
-    import fcntl
-
-    InterProcessLock = _FcntlLock
-    InterProcessReaderWriterLock = _FcntlInterProcessReaderWriterLock
-
-
-def interprocess_write_locked(path):
-    """Acquires & releases an interprocess read lock around the call into
-    the decorated function"""
+def interprocess_write_locked(path: Union[Path, str]):
+    """Acquires & releases an interprocess  **write** lock around the call into
+    the decorated function
 
+    Args:
+        path: Path to the file used for locking.
+    """
     lock = InterProcessReaderWriterLock(path)
 
     def decorator(f):
-        @six.wraps(f)
+        @functools.wraps(f)
         def wrapper(*args, **kwargs):
             with lock.write_lock():
                 return f(*args, **kwargs)
@@ -542,14 +418,17 @@ def interprocess_write_locked(path):
     return decorator
 
 
-def interprocess_read_locked(path):
-    """Acquires & releases an interprocess read lock around the call into
-    the decorated function"""
+def interprocess_read_locked(path: Union[Path, str]):
+    """Acquires & releases an interprocess **read** lock around the call into
+    the decorated function
 
+    Args:
+        path: Path to the file used for locking.
+    """
     lock = InterProcessReaderWriterLock(path)
 
     def decorator(f):
-        @six.wraps(f)
+        @functools.wraps(f)
         def wrapper(*args, **kwargs):
             with lock.read_lock():
                 return f(*args, **kwargs)
@@ -559,14 +438,17 @@ def interprocess_read_locked(path):
     return decorator
 
 
-def interprocess_locked(path):
-    """Acquires & releases a interprocess lock around call into
-       decorated function."""
+def interprocess_locked(path: Union[Path, str]):
+    """Acquires & releases an interprocess lock around the call to the
+    decorated function.
 
+    Args:
+        path: Path to the file used for locking.
+   """
     lock = InterProcessLock(path)
 
     def decorator(f):
-        @six.wraps(f)
+        @functools.wraps(f)
         def wrapper(*args, **kwargs):
             with lock:
                 return f(*args, **kwargs)
